
# q-expansion of newform 231.2.i.e, downloaded from the LMFDB on 09 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 231
weight = 2
poly_data = [9, 12, 28, -4, 21, 0, 8, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[5468, 561, 1156, -1444, 646, 357, -215, 68], 4243], [[2262, 693, 1428, 3208, 798, 441, -16, 84], 4243], [[5760, -16828, 12640, -13440, 1323, -6080, 1760, -754], 12729], [[11088, -56579, 24332, -25872, 15594, -11704, 3388, -815], 12729], [[-9252, 17165, -20303, 21588, -6978, 9766, -2827, 1052], 12729], [[-13760, -4587, -9452, -8909, -5282, -2919, 510, -556], 4243]]

hecke_ring_character_values = [[155, [1, 0, 0, 0, 0, 0, 0, 0]], [199, [0, 0, 0, 0, -1, 0, 0, 0]], [211, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, -1, 0, 0, 0, 0, 0, 0], [-1, 0, 0, 0, 1, 0, 0, 0], [0, -1, 0, 0, -1, 0, 1, 0], [0, 0, 0, -1, 0, -1, 0, -1], [1, 0, 0, 0, -1, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -1], [-2, -1, -2, -1, 2, 0, -2, 0], [0, -1, 0, 0, 0, 1, 2, 0], [0, 2, 0, 0, -1, 2, 0, 0], [2, 0, 0, 1, 0, 0, 0, 1], [1, -1, -1, -1, -1, 2, -1, 2], [0, 1, 0, 0, 0, 0, 1, 0], [0, 0, -2, 2, 0, 0, 0, 1], [4, 0, 2, -3, 0, 0, 0, 0], [0, -1, 0, 0, -2, 0, -1, 0], [3, 3, 1, 3, -3, 4, 1, 4], [-5, -2, 0, -2, 5, 2, 0, 2], [0, -2, 0, 0, 0, -1, 0, 0], [-9, -3, 1, -3, 9, 2, 1, 2], [4, 0, -1, 1, 0, 0, 0, -2], [2, 0, 6, 0, -2, -1, 6, -1], [0, 4, 0, 0, 0, 3, -2, 0], [-2, 0, 0, -2, 0, 0, 0, -4], [0, 2, 0, 0, 0, 0, -6, 0], [-10, 0, 1, -1, 0, 0, 0, -2], [-6, 1, 2, 1, 6, -1, 2, -1], [0, 2, 0, 0, 4, 4, -2, 0], [0, 6, 0, 0, -6, -3, 0, 0], [-6, -4, 0, -4, 6, 1, 0, 1], [-1, 0, -1, 7, 0, 0, 0, 0], [10, 0, 2, -1, 0, 0, 0, -3], [0, -4, 0, 0, -8, -5, 0, 0], [4, -2, 6, -2, -4, -4, 6, -4], [0, 0, 2, -5, 0, 0, 0, -2], [0, 1, 0, 0, -2, 2, 4, 0], [-12, -5, 4, -5, 12, 2, 4, 2], [5, 4, -4, 4, -5, -2, -4, -2], [0, -3, 0, 0, 3, 4, -1, 0], [14, 0, 2, 4, 0, 0, 0, 0], [0, 2, 0, 0, -6, -1, 2, 0], [-4, 3, -5, 3, 4, 2, -5, 2], [-6, 0, -4, 6, 0, 0, 0, 4], [0, -4, 0, 0, 0, -2, 4, 0], [-6, -8, -2, -8, 6, -4, -2, -4], [-2, 0, -8, 3, 0, 0, 0, 6], [14, -4, 0, -4, -14, 2, 0, 2], [-6, 0, -8, 6, 0, 0, 0, 3], [-13, 0, 3, -5, 0, 0, 0, 6], [8, 0, 4, 0, -8, -4, 4, -4], [0, 4, 0, 0, -4, 0, -10, 0], [0, 1, 0, 0, -2, -1, -2, 0], [8, 0, 2, 2, 0, 0, 0, 3], [8, 6, -2, 6, -8, 0, -2, 0], [-4, 0, 7, -9, 0, 0, 0, 2], [0, 8, 0, 0, -6, -6, 0, 0], [-8, 4, -4, 4, 8, -2, -4, -2], [-7, -1, 5, -1, 7, 6, 5, 6], [0, 4, 0, 0, -8, -1, 2, 0], [6, 2, 2, 2, -6, 0, 2, 0], [4, 0, 8, -1, 0, 0, 0, -5], [14, 6, 8, 6, -14, 1, 8, 1], [-4, 0, 0, -9, 0, 0, 0, 1], [10, 0, -4, 10, 0, 0, 0, -3], [11, -4, 6, -4, -11, -2, 6, -2], [0, -3, 0, 0, 14, -2, -3, 0], [0, 5, 0, 0, 11, 0, 1, 0], [0, -2, 0, 0, 0, -6, 4, 0], [-2, 0, -6, -4, 0, 0, 0, -2], [-14, -4, 2, -4, 14, 1, 2, 1], [-14, 0, 2, 4, 0, 0, 0, -3], [5, 1, 1, 1, -5, 2, 1, 2], [0, -2, 0, 0, 0, 9, 0, 0], [13, 3, 7, 3, -13, -2, 7, -2], [0, -4, 0, 0, 0, 5, 6, 0], [-21, 0, -1, 1, 0, 0, 0, 4], [0, -6, 0, 0, 5, 4, 8, 0], [24, 14, 6, 14, -24, 0, 6, 0], [0, 3, 0, 0, -2, 0, 9, 0], [0, 5, 0, 0, 9, 2, -1, 0], [0, -12, 0, -12, 0, -11, 0, -11], [8, 0, 1, 3, 0, 0, 0, 2], [9, 0, 8, 6, 0, 0, 0, 4], [-6, 8, 2, 8, 6, 5, 2, 5], [-14, 0, 3, 1, 0, 0, 0, -2], [0, 7, 0, 0, 10, 6, 0, 0], [0, 4, 0, 0, -5, 10, 6, 0], [-12, 0, -8, -4, 0, 0, 0, 4], [0, -4, 0, 0, -6, 5, 0, 0], [-4, 0, 4, -3, 0, 0, 0, 2], [2, 0, 6, -6, 0, 0, 0, 6], [0, -14, 0, 0, 11, 6, -2, 0], [-6, -12, 0, -12, 6, 0, 0, 0], [-21, 13, -9, 13, 21, 4, -9, 4], [0, 0, 8, -8, 0, 0, 0, -1], [0, -17, 0, 0, -3, 0, 7, 0], [-14, 0, -14, 0, 0, 0, 0, 3], [0, 0, 0, 0, -18, -4, 2, 0], [-12, 6, -8, 6, 12, -2, -8, -2], [0, 14, 0, 0, 0, 1, -4, 0], [0, 6, 0, 0, 0, -3, -8, 0], [-8, 0, -10, 13, 0, 0, 0, 3], [-4, -5, 0, -5, 4, 4, 0, 4], [4, 8, -4, 8, -4, 7, -4, 7], [0, -7, 0, 0, -8, -2, 6, 0], [-14, -3, 0, -3, 14, 3, 0, 3], [-10, -18, 8, -18, 10, 4, 8, 4], [2, 0, -10, 0, 0, 0, 0, 6], [0, -1, 0, 0, 8, 5, 0, 0], [14, 8, 2, 8, -14, 0, 2, 0], [-2, 0, 8, -2, 0, 0, 0, 6], [0, 0, 0, 0, 16, 1, 10, 0], [-4, -2, -4, -2, 4, 4, -4, 4], [13, 0, 9, 9, 0, 0, 0, -4], [3, 9, -7, 9, -3, 6, -7, 6], [-11, 0, -7, -7, 0, 0, 0, 0], [-9, -9, 3, -9, 9, -6, 3, -6], [14, 0, 4, 2, 0, 0, 0, 4], [8, -8, 4, -8, -8, -4, 4, -4], [0, -14, 0, 0, 10, 0, -4, 0], [-2, 0, 0, -14, 0, 0, 0, -10], [-5, -10, -2, -10, 5, 4, -2, 4], [4, 0, -4, 0, 0, 0, 0, -15], [0, 9, 0, 0, -4, 4, -10, 0], [-8, -3, 3, -3, 8, -4, 3, -4], [0, 2, 0, 0, -4, -8, 0, 0], [26, 0, 2, 3, 0, 0, 0, 0], [0, 5, 0, 0, 8, 2, -5, 0], [0, 11, 0, 0, -8, -6, -7, 0], [10, 0, -6, 10, 0, 0, 0, -2], [0, -6, 0, 0, -24, 1, 8, 0], [26, 16, 6, 16, -26, -1, 6, -1], [38, 0, 2, 8, 0, 0, 0, 0], [0, 10, 0, 0, -8, -10, 2, 0], [5, 0, 12, 8, 0, 0, 0, 6], [0, -12, 0, 0, 14, 3, -8, 0], [10, 0, 8, -6, 0, 0, 0, 6], [-6, -4, 4, -4, 6, -2, 4, -2], [10, -3, -4, -3, -10, -4, -4, -4], [20, 0, 8, 6, 0, 0, 0, 6], [18, -8, -4, -8, -18, -1, -4, -1], [4, 0, -2, 16, 0, 0, 0, 5], [0, 0, 0, 0, -14, -3, 8, 0], [-5, -7, 7, -7, 5, 4, 7, 4], [-16, 0, 10, -2, 0, 0, 0, 8], [-16, -3, 3, -3, 16, -4, 3, -4], [-4, 0, 0, 2, 0, 0, 0, 16], [-22, 0, -14, -12, 0, 0, 0, -8], [12, 11, 0, 11, -12, 3, 0, 3], [0, -4, 0, 0, 14, -6, -4, 0], [0, -12, 0, 0, -18, -4, 14, 0], [0, 6, 0, 0, -12, 3, -2, 0], [-12, 0, 4, -20, 0, 0, 0, 4], [16, 0, 6, 10, 0, 0, 0, 4], [0, -4, 0, 0, 8, -11, -10, 0], [13, -13, 19, -13, -13, -8, 19, -8], [10, 0, -1, 13, 0, 0, 0, -2], [0, 11, 0, 0, -32, 7, 4, 0], [0, -13, 0, 0, 17, -10, -9, 0], [6, 0, 14, -8, 0, 0, 0, -8], [-12, -7, 6, -7, 12, 3, 6, 3], [0, -11, 0, 0, -24, -4, -1, 0], [0, 0, -2, -6, 0, 0, 0, -17], [20, 0, -4, -7, 0, 0, 0, 5], [0, 10, 0, 0, 16, 8, 4, 0], [-15, -1, -1, -1, 15, 14, -1, 14], [-23, 2, 2, 2, 23, 4, 2, 4], [-20, 2, 4, 2, 20, -2, 4, -2], [-12, 16, -10, 16, 12, 0, -10, 0]]
